(function (global) {
	var Images = function () {
	};

	Images.prototype = {
		imageFilenames: ["dsc_6001.jpg", "dsc_6081.jpg", "dsc_6013.jpg", "dsc_6268.jpg", "dsc_6397.jpg", "dsc_6345.jpg", "dsc_6378.jpg", "dsc_6413.jpg", "dsc_6417.jpg"],
		isMobile: (navigator.userAgent.match(/iPad|iPhone|android/i) !== null),
		grids: [],
		startPosition: 0,
		endPosition: 0,
		sectionName: '',
		target: '',
		endTarget: '',
		targetData: '',
		endTargetData: '',
		/**
			@public
			@description gallery Adds ordered list containing image thumbnails within a specified element id (galleryId).
			@param {String} galleryId The unique id of the gallery container.
		*/
		gallery: function (galleryId) {
			var i,
				columnHolder,
				gallerySelector = '.columns .column',
				gallery;

			if (typeof galleryId === 'string') {
				try { gallery = document.getElementById(galleryId); } catch (e) {}
				if (gallery !== null) {
					// Create unqiue tracking sectionsName
					this.sectionName = "#" + galleryId;
					// append to gallery container
					gallery.innerHTML = '<ol class="columns"></ol>';
					// Find all DOM elements with a id='galleryId' and class of 'columns' and add them to a nodeList.
					try { columnHolder = document.querySelectorAll('#' + galleryId + ' .columns'); } catch (e) {}
					// Build the image gallery within the 'galleryId'
					for (i = 0; i < this.imageFilenames.length; i += 1) {
						columnHolder[0].innerHTML += '<li class="column" draggable="true"  data-name="' + this.imageFilenames[i] + '"  data-position="' + i + '"><img alt="' + this.imageFilenames[i] + '" src="images/' + this.imageFilenames[i] + '"></li>';
					}
					// Get nodeList of all list items by within gallerySelector.
					this.grids = document.querySelectorAll(gallerySelector);
					// Attach event listners
					if (global.Images.isMobile === true) {
						this.mobileDragHandler();
					} else {
						this.dragHandler();
					}
				}
			}
		},
		/**
			@public
			@description dragHandler Loops through a nodeList and sets a event listner to each list item (li).
		*/
		dragHandler: function () {
			var i;

			for (i = 0; i < this.grids.length; i += 1) {
				this.grids[i].addEventListener('dragstart', global.Images.dragStart, false);
				this.grids[i].addEventListener('dragover', global.Images.dragOver, false);
				this.grids[i].addEventListener('drop', global.Images.dragDrop, false);
				this.grids[i].addEventListener('dragend', global.Images.dragEnd, false);
			}
		},
		mobileDragHandler: function () {
			var i;

			for (i = 0; i < this.grids.length; i += 1) {
				this.grids[i].addEventListener("touchstart", global.Images.touchStart, false);
				this.grids[i].addEventListener("touchmove", global.Images.touchMove, false);
				this.grids[i].addEventListener("touchend", global.Images.touchEnd, false);
			}
		},
		/**
			@public
			@description dragStart Changes opacity of the object and saves the object's image.
			@param {Object} event Object assigned to the dragstart' event.
		*/
		dragStart: function (event) {
			if (typeof event === 'object') {
				this.style.opacity = '0.4';
				// reset endTarget local var
				global.Images.endTarget = '';
				// Set current StartElement
				global.Images.target = this;
				// Get/Set current position
				global.Images.dragStartPosition = parseInt(global.Images.getPosition(this), 10);
				// Set data
				global.Images.targetData = this.innerHTML;
				// tracking code
				global.Images.trackingCall('start', global.Images.getImageName(global.Images.target));
			}
			return false;
		},
		/**
			@public
			@description touchStart Used on mobile device. Changes opacity of the object and saves the object's image.
			@param {Object} event Object assigned to the 'touchstart' event.
		*/
		touchStart: function (event) {
			this.style.opacity = '0.4';
			// Set data
			global.Images.targetData = event.target.src;
			global.Images.target = this;
			// Get/Set current position
			global.Images.dragStartPosition = parseInt(global.Images.getPosition(this), 10);
			// tracking code
			global.Images.trackingCall('start', global.Images.getImageName(global.Images.target));
		},
		/**
			@public
			@description dragOver Prevents browser from opening the image file on drag.
			@param {Object} event Object assigned to the 'dragover' event.
		*/
		dragOver: function (event) {
			if (typeof event === 'object') {
				if (event.preventDefault) {
					event.preventDefault();
				}
			}
			return false;
		},
		/**
			@public
			@description touchMove Used on mobile device. Prevents browser from opening the image file on drag.
			@param {Object} event Object assigned to the 'touchmove' event.
		*/
		touchMove: function (event) {
			event.preventDefault();
		},
		/**
			@public
			@description dragDrop Prevents the browser from showing the full image on 'drop', and replaces the 'dragend' element's data with 'dragstart' element's data.
			@param {Object} event Object assigned to the 'drop' event.
		*/
		dragDrop: function (event) {
			if (typeof event === 'object') {
				if (event.stopPropagation) {
					event.stopPropagation();
				}

				if (event.preventDefault) {
					event.preventDefault();
				}

				if (this.innerHTML !== global.Images.targetData) {
					// gets end data and saves  to start data
					global.Images.endTargetData = this.innerHTML;
					this.innerHTML = global.Images.targetData;
					// Get/Set current position
					global.Images.dragEndPosition = parseInt(global.Images.getPosition(this), 10);
					global.Images.endTarget = this;
				}
			}
			return false;
		},
		/**
			@public
			@description touchEnd Used on mobile device. This method sets opacity of the 'touchstart' object back to 1, and replaces the 'touchend' element's image source with 'touchstart' element's image source.
			@param {Object} event Object assigned to the 'touchend' event.
		*/
		touchEnd: function (event) {
			var alt;

			global.Images.endTarget = document.elementFromPoint(
		        event.changedTouches[0].pageX,
		        event.changedTouches[0].pageY
		    );

		    global.Images.endTargetData = document.elementFromPoint(
		        event.changedTouches[0].pageX,
		        event.changedTouches[0].pageY
		    ).src;

		    alt = global.Images.endTarget.alt;

		    if (global.Images.targetData !== global.Images.endTargetData) {
		    	global.Images.endTarget.src = global.Images.targetData;
				this.innerHTML = "<img src='" + global.Images.endTargetData + "' alt='" + alt + "'>";
				// Get/Set current position
				global.Images.dragEndPosition = parseInt(global.Images.getPosition(global.Images.endTarget.parentNode), 10);
				// tracking code for END
				global.Images.trackingCall('end', global.Images.endTarget.alt);
				// save position
				global.Images.savePosition();
		    }else{
		    	// tracking code for END
				global.Images.trackingCall('cancel', undefined);
		    }

			this.style.opacity = '1';

		},
		/**
			@public
			@description dragEnd Sets opacity of the 'dragstart' object back to 1, and replaces the 'dragend' element's data with 'dragstart' element's data.
			@param {Object} event Object assigned to the 'dragend' event.
		*/
		dragEnd: function (event) {
			this.style.opacity = '1';
			// move data from this object to the previous one
			if (global.Images.endTarget !== '' && global.Images.target.innerHTML !== global.Images.endTargetData) {
				// Switch data
				global.Images.target.innerHTML = global.Images.endTargetData;  // end data
				// tracking code for END
				global.Images.trackingCall('end', global.Images.imageFilenames[global.Images.dragEndPosition]);
				// save array
				global.Images.savePosition();
			} else {
				// tracking code for CANCEL
				global.Images.trackingCall('cancel', undefined);
			}
			global.Images.cleanUp();

			return false;
		},
		/**
			@public
			@description cleanUp Utility method that loops through the image gallery and removes the 'over' class.
		*/
		cleanUp: function () {
			var i;
			for (i = 0; i < global.Images.grids.length; i += 1) {
				global.Images.grids[i].classList.remove('over');
			}
		},
		/**
			@public
			@description getImageName Returns the 'data-name' attribute value of a object.
			@param {Object} element Object containing 'data-name' attribute.
		*/
		getImageName: function (element) {
			if (typeof element === 'object') {
				var name;
				try { name = element.getAttribute('data-name'); } catch (e) {}
				return name;
			} else {
				return undefined;
			}
		},
		/**
			@public
			@description getPosition Returns the 'data-position' attribute value of a object.
			@param {Object} element Object containing 'data-position' attribute.
		*/
		getPosition: function (element) {
			if (typeof element === 'object') {
				var result;
				try { result = element.getAttribute('data-position'); } catch (e) {}
				return result;
			}
		},
		/**
			@public
			@description savePosition Saves the drag position of interaction.
		*/
		savePosition: function () {
			var startImage,
				endImage;
			// flip-flop array values
			startImage = global.Images.imageFilenames[global.Images.dragStartPosition];
			endImage = global.Images.imageFilenames[global.Images.dragEndPosition];

			// Save changes to the global.Images.imageFilenames array
			global.Images.imageFilenames[global.Images.dragStartPosition] = endImage;
			global.Images.imageFilenames[global.Images.dragEndPosition] = startImage;

			// Insert AJAX code here to save the state of the array to the server.
			//console.log(global.Images.imageFilenames);
		},
		/**
			@public
			@description trackingCall Sends draging interaction to api.js
			@param {String} interactionName The type of interaction that took place; or what the user did.
			@param {String} interactionTarget optional; A unique identifier describing the main element involved with this interaction.
		*/
		trackingCall: function (interactionName, interactionTarget) {
			switch(interactionName) {
				case 'start':
					try { analyticsAPI.post(global.Images.sectionName, 'start', interactionTarget) } catch (e) {}
				break;
				case 'end':
					try { analyticsAPI.post(global.Images.sectionName, 'end', interactionTarget) } catch (e) {}
				break;
				case 'cancel':
					try { analyticsAPI.post(global.Images.sectionName, 'cancel', undefined); } catch (e) {}
				break;
				default:
					try { analyticsAPI.post(global.Images.sectionName, 'cancel', undefined); } catch (e) {}
			}
		}
	};

	// Expose singleton instance to global namespace
	global.Images = new Images();
}(window));